@@ -35,9 +35,6 @@ except:
 
 from xoutil.collections import defaultdict
 
-__docstring_format__ = 'rst'
-__author__ = 'manu'
-
 
 class TestCollections(unittest.TestCase):
     def test_defaultdict(self):
@@ -184,6 +181,39 @@ class TestChainMap(unittest.TestCase):
         self.assertEqual(dict(d), dict(a=1, b=2, c=30))
         self.assertEqual(dict(d.items()), dict(a=1, b=2, c=30))
 
+    def test_new_child(self):
+        'Tests for changes for issue #16613.'
+        c = ChainMap()
+        c['a'] = 1
+        c['b'] = 2
+        m = {'b':20, 'c': 30}
+        d = c.new_child(m)
+        self.assertEqual(d.maps, [{'b':20, 'c':30}, {'a':1, 'b':2}])  # check internal state
+        self.assertIs(m, d.maps[0])
+
+        # Use a different map than a dict
+        class lowerdict(dict):
+            def __getitem__(self, key):
+                if isinstance(key, str):
+                    key = key.lower()
+                return dict.__getitem__(self, key)
+            def __contains__(self, key):
+                if isinstance(key, str):
+                    key = key.lower()
+                return dict.__contains__(self, key)
+
+        c = ChainMap()
+        c['a'] = 1
+        c['b'] = 2
+        m = lowerdict(b=20, c=30)
+        d = c.new_child(m)
+        self.assertIs(m, d.maps[0])
+        for key in 'abc':                                  # check contains
+            self.assertIn(key, d)
+        for k, v in dict(a=1, B=20, C=30, z=100).items():  # check get
+            self.assertEqual(d.get(k, 100), v)
+
+
 
 class TestCounter(unittest.TestCase):
 
@@ -576,12 +606,19 @@ class TestOrderedDict(unittest.TestCase):
         self.assertTrue(all(type(pair)==list for pair in od.__reduce__()[1]))
 
     def test_reduce_not_too_fat(self):
+        import sys
         # do not save instance dictionary if not needed
         pairs = [('c', 1), ('b', 2), ('a', 3), ('d', 4), ('e', 5), ('f', 6)]
         od = OrderedDict(pairs)
-        self.assertEqual(len(od.__reduce__()), 2)
+        if sys.version_info >= (3, 4):
+            self.assertIsNone(od.__reduce__()[2])
+        else:
+            self.assertEqual(len(od.__reduce__()), 2)
         od.x = 10
-        self.assertEqual(len(od.__reduce__()), 3)
+        if sys.version_info >= (3, 4):
+            self.assertIsNotNone(od.__reduce__()[2])
+        else:
+            self.assertEqual(len(od.__reduce__()), 3)
 
     def test_repr(self):
         od = OrderedDict([('c', 1), ('b', 2), ('a', 3)])
@@ -667,6 +704,24 @@ class TestOrderedDict(unittest.TestCase):
         self.assertEqual(list(MyOD(items).items()), items)
 
 
+def test_abcs():
+    from xoutil.collections import Container
+    from xoutil.collections import Iterable
+    from xoutil.collections import Iterator
+    from xoutil.collections import Sized
+    from xoutil.collections import Callable
+    from xoutil.collections import Sequence
+    from xoutil.collections import MutableSequence
+    from xoutil.collections import Set
+    from xoutil.collections import MutableSet
+    from xoutil.collections import Mapping
+    from xoutil.collections import MutableMapping
+    from xoutil.collections import MappingView
+    from xoutil.collections import ItemsView
+    from xoutil.collections import KeysView
+    from xoutil.collections import ValuesView
+
+
 if __name__ == "__main__":
     #import sys;sys.argv = ['', 'Test.testName']
     unittest.main(verbosity=2)
