@@ -33,8 +33,6 @@ from re import compile as _rcompile
 from xoutil.fs.path import normalize_path
 from xoutil.six import string_types
 
-__py33 = sys.version_info >= (3, 3)
-
 
 re_magic = _rcompile('[*?[]')
 has_magic = lambda s: re_magic.search(s) is not None
@@ -443,7 +441,7 @@ def walk_up(start, sentinel):
     return current if found else None
 
 
-if not __py33:
+if sys.version_info < (3, 4, 1):
     def makedirs(name, mode=0o777, exist_ok=False):
         """makedirs(path [, mode=0o777][, exist_ok=False])
 
@@ -455,68 +453,112 @@ if not __py33:
         raised.  This is recursive.
 
         """
-        import stat as st
-        from xoutil.string import safe_encode
-        def _get_masked_mode(mode):
-            mask = os.umask(0)
-            os.umask(mask)
-            return mode & ~mask
         import errno
-        head, tail = os.path.split(name)
+        from os import path, mkdir
+        from os.path import curdir
+        from xoutil.string import safe_encode
+        head, tail = path.split(name)
         if not tail:
-            head, tail = os.path.split(head)
-        if head and tail and not os.path.exists(head):
+            head, tail = path.split(head)
+        if head and tail and not path.exists(head):
             try:
                 makedirs(head, mode, exist_ok)
             except OSError as e:
                 # be happy if someone already created the path
                 if e.errno != errno.EEXIST:
                     raise
-            cdir = os.curdir
-            if isinstance(tail, str):
-                cdir = safe_encode(os.curdir, 'ASCII')
-            if tail == cdir:     # xxx/newdir/. exists if xxx/newdir exists
+            cdir = curdir
+            if isinstance(tail, bytes):
+                cdir = safe_encode(curdir, 'ASCII')
+            if tail == cdir:      # xxx/newdir/. exists if xxx/newdir exists
                 return
         try:
-            os.mkdir(name, mode)
+            mkdir(name, mode)
         except OSError as e:
-            dir_exists = os.path.isdir(name)
-            expected_mode = _get_masked_mode(mode)
-            if dir_exists:
-                # S_ISGID is automatically copied by the OS from parent to
-                # child directories on mkdir.  Don't consider it being set to
-                # be a mode mismatch as mkdir does not unset it when not
-                # specified in mode.
-                actual_mode = st.S_IMODE(lstat(name).st_mode) & ~st.S_ISGID
-            else:
-                actual_mode = -1
-            if not (e.errno == errno.EEXIST and exist_ok and dir_exists and
-                    actual_mode == expected_mode):
-                if dir_exists and actual_mode != expected_mode:
-                    e.strerror += ' (mode %o != expected mode %o)' % (
-                        actual_mode, expected_mode)
+            if not exist_ok or e.errno != errno.EEXIST or not path.isdir(name):
                 raise
 else:
     from os import makedirs
-del __py33
 
 
-def ensure_filename(filename):
+def ensure_filename(filename, yields=False):
     '''Ensures the existence of a file with a given filename.
 
     If the filename is taken and is not pointing to a file (or a link to a
-    file) an OSError is raised.
+    file) an OSError is raised.  If `exist_ok` is False the filename must not
+    be taken; an OSError is raised otherwise.
 
     The function creates all directories if needed. See :func:`makedirs` for
     restrictions.
 
+    If `yields` is True, returns the file object.  This way you may open a
+    file for writing like this::
+
+      with ensure_filename('/tmp/good-name-87.txt', yields=True) as fh:
+          fh.write('Do it!')
+
+    The file is open in mode 'w+b'.
+
+    .. versionadded:: 1.6.1  Added parameter `yield`.
+
     '''
     if not os.path.exists(filename):
         filename = normalize_path(filename)
         dirname = os.path.dirname(filename)
         makedirs(dirname, exist_ok=True)
-        open(filename, 'wb').close()
+        # TODO: Better hanlding of mode for reading/writing.
+        fh = open(filename, 'w+b')
+        if not yields:
+            fh.close()
+        else:
+            return fh
     else:
         if not os.path.isfile(filename):
             raise OSError('Expected a file but another thing is found \'%s\'' %
                           filename)
+
+
+def concatfiles(*files):
+    '''Concat several files to a single one.
+
+    Each positional argument must be either:
+
+    - a file-like object (ready to be passed to :func:`shutil.copyfileobj`)
+
+    - a string, the file path.
+
+    The last positional argument is the target.  If it's file-like object it
+    must be open for writing, and the caller is the responsible for closing
+    it.
+
+    Alternatively if there are only two positional arguments and the first is
+    a collection, the sources will be the members of the first argument.
+
+    '''
+    import shutil
+    from xoutil.types import is_collection
+    from xoutil.six import string_types
+    if len(files) < 2:
+        raise TypeError('At least 2 files must be passed to concatfiles.')
+    elif len(files) == 2:
+        files, target = files[0], files[1]
+        if not is_collection(files):
+            files = [files]
+    else:
+        files, target = files[:-1], files[-1]
+    if isinstance(target, string_types):
+        target, opened = open(target, 'wb'), True
+    else:
+        opened = False
+    try:
+        for f in files:
+            if isinstance(f, string_types):
+                fh = open(f, 'rb')
+            else:
+                fh = f
+            shutil.copyfileobj(fh, target)
+    finally:
+        if opened:
+            target.close()
+
+del sys
